#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Mon Mar 11 2024

@auteur: David ALBERTO
(www.astrolabe-science.fr)

Ce script calcule l'équation du temps pour une année entière (une valeur
par jour), d'après une formule issue de 
Denis Savoie, La Gnomonique (Les Belles Lettres).
D. Savoie indique une erreur maximale de 30 secondes sur la période 1950-2050.
Pour 2024, l'écart avec les données calculées par l'imcce est de 
moins de 13 secondes.
"""
import matplotlib.pyplot as plt
import numpy as np
from datetime import datetime, timedelta
from math import sin, cos, atan2, radians, degrees

import matplotlib.dates as mdates # formatage de l'axe de dates
import locale # noms des mois en français
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français

annee = 2025
nom_fichier = "equation_temps" + str(annee)

eps = radians(23.439) # obliquité de l'écliptique
plt.rcParams["font.family"] = "Roboto Condensed" # ou autre police installée
plt.rcParams["font.size"] = 8

# création d'une liste de dates (tous les jours de l'année):
t = np.arange(datetime(annee,1,1,12), datetime(annee+1,1,1,12),
              timedelta(days=1)).astype(datetime)

def equation_temps(date):
    """
    Calcule l'équation du temps pour une date donnée.
    D'après D. Savoie (Gnomonique) Précision env.30 s pour 1950-2050.
    Parameters
    ----------
    date : TYPE : date au format datetime : datetime(annee, mois , jour)
        DESCRIPTION : date du jour souhaité.
    Returns
    -------
    Renvoie la valeur de l'équation du temps à la date demandée,
    sous forme de tuple : (valeur décimale, string en format min:sec).
    """
    Tzero = datetime(2000, 1, 1, 12) #  1er janvier 2000 12hTU, date de référence
    T = (date - Tzero).days
    T = T + (date - Tzero).seconds / 3600 / 24
    LM = (280.460 + 0.9856474 * T) % 360
    M = (357.528 + 0.9856003 * T) % 360
    long = (LM + 1.915 * sin(radians(M)) + 0.020 * sin(radians(2*M))) % 360
    AD = atan2(cos(eps) * sin(radians(long)), cos(radians(long)))
    EdT_degres = ((degrees(AD) - LM))%360
    if EdT_degres > 180:
        EdT_degres = EdT_degres - 360
    EdT = EdT_degres * 4
    return EdT

def minimum(date1, date2):
    """
    Parameters
    ----------
    date1 : TYPE : date datetime
        DESCRIPTION.
    date2 : TYPE date datetime
        DESCRIPTION.
    Returns
    -------
    Cherche le minimum local et l'affiche.
    """
    periode = np.arange(date1, date2, timedelta(days=1)).astype(datetime)
    mini = EQT(date1)
    for date in periode:
        if EQT(date) < mini:
            mini = EQT(date)
            jour = date
    ax.scatter(jour, mini, s=10, c='white', ec='r', alpha=1, zorder=2)
    ax.text(jour, mini-1, mmss(mini), ha='center', va='center', c='r')
    ax.text(jour, mini+1.4, jour.strftime('%d %b'), ha='center', va='center', c='r')

def maximum(date1, date2):
    """
    Parameters
    ----------
    date1 : TYPE : date datetime
        DESCRIPTION.
    date2 : TYPE date datetime
        DESCRIPTION.
    Returns
    -------
    Cherche le maximum local et l'affiche.
    """
    periode = np.arange(date1, date2, timedelta(days=1)).astype(datetime)
    maxi = EQT(date1)
    for date in periode:
        if EQT(date) > maxi:
            maxi = EQT(date)
            jour = date
    ax.scatter(jour, maxi, s=10, c='white', ec='slateblue', alpha=1, zorder=2)
    ax.text(jour, maxi+0.8, mmss(maxi), ha='center', va='center', c='slateblue')
    ax.text(jour, maxi-1.4, jour.strftime('%d %b'), ha='center', va='center', c='slateblue')

def mmss(M):
    """
    Transforme les minutes décimales en format sexagésimal.
    Parameters
    ----------
    H : TYPE : float
        DESCRIPTION : heure en format décimal.
    -------
    Returns : temps en format string (min:secondes)
    """
    minutes = int(M)
    secondes = round((M-minutes)*60)
    if secondes == 60:
        secondes = '00'
        minutes = minutes + 1
    elif abs(secondes) <10:
        secondes = '0' + str(secondes)
    if secondes < 0:
        secondes = - secondes
    temps_60 = f"{minutes}min{secondes}s"
    return temps_60

def zero(date1, date2):
    """
    Recherche la date d'un zéro de l'équation du temps, et l'affiche.
    """
    periode = np.arange(date1, date2, timedelta(days=1)).astype(datetime)
    for N, jour in enumerate(periode[:-1]):
        if EQT(periode[N+1]) / EQT(periode[N]) < 0:
            ax.scatter(jour, 0, s=10, c='white', ec='k', alpha=1, zorder=2)
            ax.text(jour-timedelta(days=2), 0, jour.strftime('%d %b'), ha='right', va='center', c='k')

# vectorisation de la fonction equation_temps, pour qu'elle puisse accepter
# un array numpy en argument (et non un float unique).
EQT = np.vectorize(equation_temps)

edt_calc = EQT(t)# array contenant les valeurs calculée

fig = plt.figure(tight_layout = True)
ax = plt.subplot(111)
ax.set_xlim(t[0], t[-1])
ax.set_yticks(np.arange(-16,16,2))

# formattage des graduations de dates :
locator1 = mdates.AutoDateLocator()
formatter = mdates.ConciseDateFormatter(locator1)
jours = mdates.DayLocator(bymonthday=(10,20))#graduation secondaire tous les 10 jours
# ax.xaxis.set_major_locator(locator1)
ax.xaxis.set_minor_locator(jours)
# ax.xaxis.set_major_formatter(formatter)
# ax.xaxis.set_minor_formatter(formatter)
ax.xaxis.set_major_formatter(
    mdates.ConciseDateFormatter(ax.xaxis.get_major_locator()))

ax.text(0.5, 0.90, "équation du temps (min)", transform = ax.transAxes,
        ha='center', fontsize=14, bbox=dict(boxstyle='round', facecolor='linen'))
ax.grid()

ax.plot(t,edt_calc, c='indigo')

# recherche des minima :
minimum(datetime(annee,4,1), datetime(annee,7,1))
minimum(datetime(annee,10,1), datetime(annee,11,5))
# recherche des maxima :
maximum(datetime(annee,1,1), datetime(annee,3,5))
maximum(datetime(annee,7,1), datetime(annee,9,5))
# recherche des zéros :
zero(datetime(annee,4,1), datetime(annee,4,30))
zero(datetime(annee,5,30), datetime(annee,6,30))
zero(datetime(annee,8,15), datetime(annee,9,5))
zero(datetime(annee,12,15), datetime(annee,12,31))

fig.savefig(nom_fichier + ".pdf")
fig.savefig(nom_fichier + ".png", dpi=250)
